from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager
import time


with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log.csv") as dev:
    dev.zero_position() # has a delay!
    time.sleep(1.5)
    # dev.set_current_gains()
    loop = SoftRealtimeLoop(dt=0.01, report=True, fade=0.0)
    for t in loop:
        dev.update()
        print("\r" + str(dev) + "    Temp: " + str(dev.T) + "C   Error: " + str(dev.get_motor_error_code()),end='')


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
import time
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3

def read_only(dev):
    dev.zero_position()
    time.sleep(1.5) # wait for the motor to zero (~1 second)
    
    print("Starting read only demo. Press ctrl+C to quit.")
    loop = SoftRealtimeLoop(dt=0.01, report=True, fade=0.0)
    
    for t in loop:
        dev.update()
        print("\r" + str(dev), end='')


if __name__ == '__main__':
    with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file="log.csv") as dev:
        read_only(dev)


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
from NeuroLocoMiddleware.SysID import Chirp
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager

def chirp_demo(dev, amp=1.0, dt=0.001):
    print("Chirping ActPackA. Press CTRL-C to finish.")
    chirp = Chirp(250, 25, 1)
    dev.set_current_gains()
    
    loop = SoftRealtimeLoop(dt = dt, report=True, fade=0.1)
    for t in loop:
        dev.update()
        des_τ = loop.fade*amp*chirp.next(t)*3/3.7
        dev.τ = des_τ # a barely audible note
        # print(t, des_τ)

def main():
    with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log.csv",use_torque_compensation=False) as dev:
        chirp_demo(dev, amp=3.0)
    print("done with chirp_demo()")

if __name__ == '__main__':
    main()


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
from NeuroLocoMiddleware.SysID import Chirp
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3


def chirp_demo(dev, amp=1.0, dt=0.001):
    print("Chirping ActPackA. Press CTRL-C to finish.")
    chirp = Chirp(250, 25, 1)
    dev.set_current_gains()
    
    print("Starting current chirp demo. Press ctrl+C to quit.")

    loop = SoftRealtimeLoop(dt = dt, report=True, fade=0.1)
    for t in loop:
        dev.update()
        des_τ = loop.fade*amp*chirp.next(t)*3/3.7
        dev.τ = des_τ # a barely audible note
        # print(t, des_τ)

if __name__ == '__main__':
    with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file="log.csv") as dev:
        chirp_demo(dev, amp=1.0)
    print("done with chirp_demo()")


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
import time
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3

def current_step(dev):
    dev.zero_position()
    time.sleep(1.5) # wait for the motor to zero (~1 second)
    dev.set_current_gains()
    
    print("Starting current step demo. Press ctrl+C to quit.")

    loop = SoftRealtimeLoop(dt = 0.01, report=True, fade=0)
    for t in loop:
        dev.update()
        if t < 1.0:
            dev.i = 0.0
        else:
            dev.i = 0.5

    del loop

if __name__ == '__main__':
    with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file="log.csv") as dev:
        current_step(dev)


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
import numpy as np
import time
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
ID_1 = 3
ID_2 = 2

Type_1 = 'AK80-9'
Type_2 = 'AK80-9'


def two_DOF(dev1,dev2):
    dev1.zero_position()
    dev2.zero_position()
    time.sleep(1.5) # wait for the motors to zero (~1 second)
    dev1.set_impedance_gains_real_unit(K=10.0,B=0.5)
    dev2.set_impedance_gains_real_unit(K=10.0,B=0.5)
    
    print("Starting 2 DOF demo. Press ctrl+C to quit.")

    loop = SoftRealtimeLoop(dt = 0.005, report=True, fade=0)
    for t in loop:
        dev1.update()
        dev2.update()
        if t < 1.0:
            dev1.θ = 0.0
            dev2.θ = 0.0
        else:
            dev1.θ = (np.pi/2)*int(t)
            dev2.θ = (np.pi/2)*int(t)

    del loop

if __name__ == '__main__':
    # to use additional motors, simply add another with block
    # remember to give each motor a different log name!
    with TMotorManager(motor_type=Type_1, motor_ID=ID_1, CSV_file="log_dev1.csv") as dev1:
        with TMotorManager(motor_type=Type_2, motor_ID=ID_2, CSV_file="log_dev2.csv") as dev2:
            two_DOF(dev1,dev2)


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager
import time

with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log.csv") as dev:
    dev.zero_position() # has a delay!
    time.sleep(1.5)
    dev.set_impedance_gains_real_unit(K=1,B=0.1)
    
    loop = SoftRealtimeLoop(dt = 0.01, report=True, fade=0)
    for t in loop:
        dev.update()
        if t < 1.0:
            dev.θ = 0.0
        elif t < 2.0:
            dev.θ = 12.4
        elif t < 3.0:
            dev.θ = 12.6
        

    del loop


import csv
from NeuroLocoMiddleware.AdcManager import ADC_Manager
from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop

from sys import path
path.append("/home/pi/TMotorCANControl/src")
from TMotorCANControl.TMotorManager import TMotorManager

test_torques = [0.0]# [0.0, 0.25,-0.25, 1.0,-1.0,5.0,-5.0,10.0,-10.0] # Nm
csv_file_name = "log_adc_and_motor.csv"

bias = 0.82380554157 + 0.27416231671018404 - 0.5243091399609011 #  0.5896742308538513 # Nm
delay = 20.0 # s
torque = test_torques[0] # Nm

with ADC_Manager(csv_file_name="dummyLog") as adc:
    with TMotorManager(motor_ID=3,CSV_file=None,use_torque_compensation=False) as dev:
        with open(csv_file_name,'w') as fd:
            writer = csv.writer(fd)
            writer.writerow(["loop_time","motor_torque_command","adc_measured_torque","motor_measured_torque","motor_measured_current","motor_measured_speed"])
            
            dev.set_current_gains()
            loop = SoftRealtimeLoop(dt=0.01, report=True, fade=0.0)
            t_next = delay
            i = 0
            print("Testing torque: " + str(torque) + " Nm")
            for t in loop:
                if t > t_next and i < len(test_torques):
                    i += 1
                    torque = test_torques[i]
                    t_next += delay
                    print("Testing torque: " + str(torque) + " Nm")
                adc.update()
                dev.update()
                writer.writerow([t, torque, adc.get_torque() - bias, dev.τ, dev.i, dev.θd])
                dev.τ = torque
                
                
            del loop


try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR dev!
ID = 3
Type = 'AK80-9'

with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file="log.csv") as dev:
    if dev.check_can_connection():
        print("\nmotor is successfully connected!\n")
    else:
        print("\nmotor not connected. Check dev power, network wiring, and CAN bus connection.\n")
    


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager
import time

logvars = [
    "output_angle", 
    "output_velocity", 
    "output_acceleration",
    "current",
    "output_torque",
    "motor_angle", 
    "motor_velocity", 
    "motor_acceleration", 
    "motor_torque"
]

with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log.csv", log_vars=logvars) as dev:
    dev.zero_position() # has a delay!
    time.sleep(1.5)
    dev.set_impedance_gains_real_unit(K=10,B=0.5)

    loop = SoftRealtimeLoop(dt = 0.01, report=True, fade=0)
    for t in loop:
        dev.update()
        if t < 1.0:
            dev.ϕ = 0.0
        else:
            dev.ϕ = 10.0

    del loop


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
from NeuroLocoMiddleware.SysID import Chirp
import numpy as np
import time
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3

def feature_cycle(dev):
    dev.zero_position() 
    time.sleep(1.2)  # wait for the motor to zero (~1 second)
    chirp_slow = Chirp(250, 200, 1)
    loop = SoftRealtimeLoop(dt = 0.001, report=True, fade=0)
    amp = 3.0
    
    # start in current mode
    state = 0
    dev.set_current_gains() 
    
    delta = 5
    t_next=delta
    print("(1 of 8) Setting torque to 0.0 Nm")

    for t in loop:
        dev.update()
        if(t > t_next):
            state += 1
            if state == 1:
                print("(2 of 8) Chirping")
                # dev.set_current_gains()
            elif state == 2:
                print("(3 of 8) Zeroing Position K = 1.0 B =0.05")
                dev.set_impedance_gains_real_unit(K=1.0,B=0.05)
            elif state == 3:
                print("(4 of 8) Zeroing Position K = 10 B =0.5")
                dev.set_impedance_gains_real_unit(K=10.0,B=0.5)
            elif state == 4:
                print("(5 of 8) Tracking sinusoidal trajectory")
                # dev.set_impedance_gains_real_unit(K=10.0,B=0.5)
            elif state == 5:
                print("(6 of 8) Full state feedback. Chirping during impedance control!")
                dev.set_impedance_gains_real_unit_full_state_feedback(K=10.0,B=0.5)
            elif state == 6:
                print("(7 of 8) Setting torque to 0.5 Nm")
                dev.set_current_gains()
            elif state == 7:
                print("(8 of 8) Setting current to -0.5 A")
                # dev.set_current_gains()
            elif state == 8:
                print("Done! Press ctrl+C to exit.")
                # dev.set_current_gains()
                dev.i = 0.0

            t_next += delta
                
        if state == 0:
            dev.i = 0.0
        elif state == 1:
            dev.τ = loop.fade*amp*chirp_slow.next(t)*3/3.7
        elif state == 2:
            dev.θ = 0.0
        elif state == 3:
            dev.θ = 0.0
        elif state == 4:
            dev.θ = 2.0*np.sin(np.pi*t)
        elif state == 5:
            dev.θ = 1.0*np.sin(np.pi*t)
            dev.τ = loop.fade*amp*chirp_slow.next(t)*3/3.7
        elif state == 6:
            dev.τ = 0.5
        elif state == 7:
            dev.i = -0.5
        else:
            dev.i=0.0

    del loop



if __name__ == '__main__':
    with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file=None) as dev:
        feature_cycle(dev)


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop

from TMotorCANControl.CAN_Manager import MIT_Params, MIT_motor_state
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager

import numpy as np
import time
from NeuroLocoMiddleware.SysID import Chirp


# to use additional motors, simply add another with block
# remember to give each motor a different log name!
with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log_dev1.csv") as dev1:
    with TMotorManager(motor_type='AK80-9', motor_ID=2, CSV_file="log_dev2.csv") as dev2:
        dev1.zero_position() # has a delay!
        # no need to zero dev2's position because we never use it.
        time.sleep(1.5)
        dev1.set_impedance_gains_real_unit(K=10.0,B=0.5)
        dev2.set_current_gains()
        chirp = Chirp(250, 200, 0.5)
        amp = 10.0
        loop = SoftRealtimeLoop(dt = 0.005, report=True, fade=0)
        for t in loop:
            dev1.update()
            dev2.update()
            if t < 1.0:
                dev1.θ = 0.0
                dev2.τ = 0.0
            else:
                dev1.θ = (np.pi/2)*int(t)
                dev2.τ = loop.fade*amp*chirp.next(t)*3/3.7

        del loop


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop

# try to import the default version, if that fails import from local directory
try:
     from TMotorCANControl.TMotorManager import TMotorManager
except ModuleNotFoundError:
    from sys import path
    path.append("/home/pi/TMotorCANControl/src")
    from TMotorCANControl.TMotorManager import TMotorManager
import numpy as np
import time
from NeuroLocoMiddleware.SysID import Chirp

with TMotorManager(motor_type='AK80-9', motor_ID=3, CSV_file="log.csv") as dev:
    dev.zero_position() # has a delay!
    time.sleep(1.5)
    dev.set_impedance_gains_real_unit_full_state_feedback(K=10,B=1)
    chirp = Chirp(250, 200, 0.5)
    loop = SoftRealtimeLoop(dt = 0.001, report=True, fade=0)
    amp = 3.0
  
    for t in loop:
        dev.update()
        if t < 1.0:
            dev.τ = 0.0
            dev.θ = 0.0
        else:
            des_τ = loop.fade*amp*chirp.next(t)*3/3.7
            dev.τ = des_τ
            dev.θ = (np.pi/2)*int(t)

    del loop


from cProfile import label
from matplotlib import pyplot as plt
import csv
import numpy as np
from scipy.signal import butter, lfilter, freqz

def butter_lowpass(cutoff, fs, order=5):
    return butter(order, cutoff, fs=fs, btype='low', analog=False)

def butter_lowpass_filter(data, cutoff, fs, order=5):
    b, a = butter_lowpass(cutoff, fs, order=order)
    y = lfilter(b, a, data)
    return y

time = []
torque_command = []
torque_adc = []
torque_motor = []
current_motor = []
speed_motor = []

test_dir= "saved_logs/"
log_dir="torque_vary_with_angle/"
name="test-opposing-motor-2"


with open(test_dir + log_dir + name + ".csv",'r') as fd:
    reader = csv.reader(fd)
    i = 0
    for row in reader:
        if i > 1:
            time.append(float(row[0]))
            torque_command.append(float(row[1]))
            torque_adc.append(float(row[2]))
            torque_motor.append(float(row[3]))
            current_motor.append(float(row[4]))
            speed_motor.append(float(row[5]))
        i += 1

torque_adc_adjusted = -np.array(torque_adc)
# og_torque = [0.091*9*i for i in current_motor]

order = 6
fs = 1/0.01       # sample rate, Hz
cutoff = 10.0  # desired cutoff frequency of the filter, Hz

torque_adc_filtered = butter_lowpass_filter(torque_adc_adjusted, cutoff, fs, order).reshape(-1,)

# torque_adc_filtered = np.array(torque_adc_filtered)
torque_motor = np.array(torque_motor)
current_motor = np.array(current_motor)

# og_torque = np.array(og_torque)

print("Average τ_adc: " + str(np.average(torque_adc_filtered)))
print("Std Dev τ_adc: " + str(np.std(torque_adc_filtered)))
print("Max τ_adc: " + str(torque_adc_filtered.max()))

print("Average τ_motor: " + str(np.average(torque_motor)))
print("Std Dev τ_motor: " + str(np.std(torque_motor)))
print("Max τ_motor: " + str(torque_motor.max()))

# current_motor = current_motor/1.56

print("Average i_motor: " + str(np.average(current_motor)))
print("Std Dev i_motor: " + str(np.std(current_motor)))
print("Max i_motor: " + str(current_motor.max()))



# plt.subplot(2, 1, 1)
# plt.plot(np.array(time),torque_adc_adjusted.flatten(),label="τ_adc_raw (max: " + str(round(torque_adc_adjusted.max(),2)) + "Nm)")
# plt.plot(time,og_torque,label="τ_motor (max: "+ str(round(og_torque.max(),2)) + "Nm)")
# plt.plot(time,og_torque,label="τ_unadjusted (max: "+ str(round(og_torque.max(),2)) + "Nm)" )
# plt.plot(np.array(time),speed_motor,label="v")
# plt.plot(np.array(time),current_qaxis,label="i_q (max: " + str(round(current_qaxis.max(),2)) + "A)")
plt.plot(np.array(time),torque_adc_adjusted,label="τ_adc (max: " + str(round(torque_adc_adjusted.max(),2)) + "Nm)")
# plt.plot(np.array(time),torque_adc_filtered,label="τ_adc (max: " + str(round(torque_adc_filtered.max(),2)) + "Nm)")
# plt.plot(np.array(time),current_adc,label="i_q (max: " + str(round(current_adc.max(),2)) + "A)")

# plt.plot(np.array(time),current_motor,label="i_motor (max: " + str(round(current_motor.max(),2)) + "A)")
# plt.plot(np.array(time),torque_motor,label="τ_q (max: " + str(round(torque_motor.max(),2)) + "Nm)")
plt.title('Torque vs Time')
plt.ylabel('Torque [Nm]')
plt.xlabel('Time [s]')
plt.grid(True)
plt.legend()

plt.show()

plt.savefig(test_dir + log_dir + name + ".png")
# plt.clf()





from sys import path
path.append("/home/pi/TMotorCANControl/src")
from TMotorCANControl.TMotorManager import TMotorManager
from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
from NeuroLocoMiddleware.AdcManager import ADC_Manager
import csv


bias = 0.5896742308538513 # Nm
test_time = 10.0 # s
torque = 0.0 # Nm (starting torque)
max_torque = 25.0 # Nm

τ_per_sec = max_torque/test_time

csv_file_name = "current_comp_test_1.csv"
with ADC_Manager(csv_file_name="dummyLog") as adc:
    with TMotorManager(motor_ID=3,CSV_file=None,use_torque_compensation=False) as dev:
        with open(csv_file_name,'w') as fd:
            writer = csv.writer(fd)
            writer.writerow(["loop_time","motor_torque_command","adc_measured_torque","motor_measured_torque","motor_measured_current","motor_measured_speed"])
            
            dev.set_current_gains()
            loop = SoftRealtimeLoop(dt=0.01, report=True, fade=0.0)
            i = 0
            for t in loop:
                
                print("\rTesting torque: " + str(torque) + " Nm",end="")
                adc.update()
                dev.update()
                writer.writerow([t, torque, adc.get_torque() - bias, dev.τ, dev.i, dev.θd])
                if t <= test_time:
                    torque = τ_per_sec*t
                    dev.τ = torque
                else:
                    dev.τ = 0.0
                
            del loop


from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
from NeuroLocoMiddleware.SysID import Chirp
import numpy as np
import time
from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3

def full_state_feedback(dev):
    dev.zero_position() # has a delay!
    time.sleep(1.5)
    dev.set_impedance_gains_real_unit_full_state_feedback(K=10,B=1)
    chirp = Chirp(250, 200, 0.5)

    print("Starting full state feedback demo. Press ctrl+C to quit.")

    loop = SoftRealtimeLoop(dt = 0.001, report=True, fade=0)
    amp = 1.0
  
    for t in loop:
        dev.update()
        if t < 1.0:
            dev.τ = 0.0
            dev.θ = 0.0
        else:
            des_τ = loop.fade*amp*chirp.next(t)*3/3.7
            dev.τ = des_τ
            dev.θ = (np.pi/2)*int(t)

    del loop

if __name__ == '__main__':
    with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file="log.csv") as dev:
        full_state_feedback(dev)


from TMotorCANControl.TMotorManager import TMotorManager

# CHANGE THESE TO MATCH YOUR DEVICE!
Type = 'AK80-9'
ID = 3

with TMotorManager(motor_type=Type, motor_ID=ID, CSV_file=None) as dev:
    if dev.check_can_connection():
        print("\nmotor is successfully connected!\n")
    else:
        print("\nmotor not connected. Check dev power, network wiring, and CAN bus connection.\n")
    